﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Unitronics.ComDriver;
using Unitronics.ComConfig.Lite.Presenters;

namespace Unitronics.ComConfig.Lite
{
    [ToolboxBitmap(typeof(resfinder), "Unitronics.ComConfig.Lite.Resources.ToolboxBitmap.png")]
    [Designer(typeof(ComConfigDesigner))]
    public partial class ComConfig : UserControl
    {
        public delegate void DirtyBitValueChanged(bool value);
        public event DirtyBitValueChanged OnDirtyBitValueChanged;

        private float dpi;

        public enum Channels
        {
            Serial = 1,
            Ethernet = 2,
            EthernetListener = 4,
        }

        public enum eViewStyle
        {
            Office2000,
            Office2003,
            Office2007,
            VisualStudio2005,
            XP,
            XPExplorerBar,
        }

        private ComConfigPresenter m_Presenter;
        private Size m_DefaultSize;
        private bool m_DirtyBit = false;
        private bool m_ForcePlcName = false;
        private bool inCheckConnectionMode = false;

        public ComConfig()
        {
            InitializeComponent();
            Graphics graphics;
            graphics = this.CreateGraphics();
            dpi = graphics.DpiX;
            m_DefaultSize = new Size(Convert.ToInt32(438 * (dpi / 96)), Convert.ToInt32(404 * (dpi / 96)));
            this.Size = m_DefaultSize;
            m_Presenter = new ComConfigPresenter();
            List<Channels> channels = m_Presenter.SupportedChannelsList;
            loadChannelsComboFromList(channels);
            cmbConnectionType.Focus();
            favoritesView1.OnFavoriteRowLoad += new EventHandler(favoritesView1_OnFavoriteRowLoad);
            plcInformation1.OnGetPlcInformationFinished += new PlcInformation.GetPlcInformationFinishedDelegate(plcInformation1_OnGetPlcInformationFinished);
            plcInformation1.OnGetPlcInformationStart += new EventHandler(plcInformation1_OnGetPlcInformationStart);
        }

        public int SupportedChannels
        {
            get
            {
                return m_Presenter.SupportedChannels;
            }
            set
            {
                m_Presenter.SupportedChannels = value;
                List<Channels> channels = m_Presenter.SupportedChannelsList;
                favoritesView1.SupportedChannelsList = channels;
                loadChannelsComboFromList(channels);
            }
        }

        public bool DirtyBit
        {
            get
            {
                return m_DirtyBit;
            }
            internal set
            {
                m_DirtyBit = value;
                if (OnDirtyBitValueChanged != null)
                    OnDirtyBitValueChanged(m_DirtyBit);
            }
        }

        [Browsable(false)]
        public Color BackColor
        {
            get
            {
                return base.BackColor;
            }
            internal set
            {
                base.BackColor = value;
            }
        }

        [Browsable(false)]
        public Color ForeColor
        {
            get
            {
                return base.ForeColor;
            }
            internal set
            {
                base.ForeColor = value;
            }
        }

        public bool ForcePlcName
        {
            get
            {
                return m_ForcePlcName;
            }
            set
            {
                m_ForcePlcName = value;
                if (panel1.Controls.Count > 0)
                {
                    IChannelView channelView = (panel1.Controls[0] as IChannelView);
                    channelView.ForcePlcName(m_ForcePlcName);
                }
            }
        }      

        // Precent resize
        protected override void SetBoundsCore(int x, int y, int width, int height,
            BoundsSpecified specified)
        {
            base.SetBoundsCore(x, y, m_DefaultSize.Width, m_DefaultSize.Height, specified);
        }


        public void Load(string xmlFilePath)
        {
            PlcComConfig plcComConfig = m_Presenter.Load(xmlFilePath);
            for (int i = 0; i < cmbConnectionType.Items.Count; i++)
            {
                Channel channel = ((PlcComConfig)cmbConnectionType.Items[i]).Channel;
                if (channel.GetType() == plcComConfig.Channel.GetType())
                {
                    cmbConnectionType.Items[i] = plcComConfig;
                    //setChannelPresenterByPlcComConfig(plcComConfig);

                    cmbConnectionType.SelectedIndex = -1;
                    cmbConnectionType.SelectedIndex = i;

                    DirtyBit = false;

                    return;
                }
            }

            throw new ApplicationException("The communication Channel in the XML file is not one of the options of the Connection Types");
            
        }

        public void Load(PlcComConfig plcComConfig)
        {
            PlcComConfig replica = Globals.Utils.GetPlcComConfigReplica(plcComConfig);

            for (int i = 0; i < cmbConnectionType.Items.Count; i++)
            {
                Channel channel = ((PlcComConfig)cmbConnectionType.Items[i]).Channel;
                if (channel.GetType() == plcComConfig.Channel.GetType())
                {
                    cmbConnectionType.Items[i] = replica;
                    //setChannelPresenterByPlcComConfig(plcComConfig);

                    cmbConnectionType.SelectedIndex = -1;
                    cmbConnectionType.SelectedIndex = i;

                    DirtyBit = false;

                    return;
                }
            }

            throw new ApplicationException("The communication Channel in the XML file is not one of the options of the Connection Types");
        }

        public void Save(string xmlFilePath)
        {
            PlcComConfig plcComConfig = (PlcComConfig)cmbConnectionType.SelectedItem;
            m_Presenter.Save(xmlFilePath, plcComConfig);

            DirtyBit = false;
        }

        public PlcComConfig GetPlcComConfig()
        {
            PlcComConfig plcComConfig = (PlcComConfig)cmbConnectionType.SelectedItem;
            return plcComConfig;
        }

        public Channel GetChannel()
        {
            PlcComConfig plcComConfig = (PlcComConfig)cmbConnectionType.SelectedItem;
            return plcComConfig.Channel;
        }

        private void setChannelPresenterByPlcComConfig(PlcComConfig plcComConfig)
        {
            if (plcComConfig.Channel is Serial)
            {
                SerialView serialView = new SerialView(plcComConfig);
                serialView.ForcePlcName(m_ForcePlcName);
                panel1.Controls.Clear();
                panel1.Controls.Add(serialView);
                serialView.OnSetAsDirty += new EventHandler(OnSetAsDirty);
                serialView.OnCheckConnection += new EventHandler(OnCheckConnection);
                serialView.Dock = DockStyle.Fill;
            }
            else if (plcComConfig.Channel is Ethernet)
            {
                EthernetView ethernetView = new EthernetView(plcComConfig);
                ethernetView.ForcePlcName(m_ForcePlcName);
                panel1.Controls.Clear();
                panel1.Controls.Add(ethernetView);
                ethernetView.OnSetAsDirty += new EventHandler(OnSetAsDirty);
                ethernetView.OnCheckConnection += new EventHandler(OnCheckConnection);
                ethernetView.Dock = DockStyle.Fill;
            }
            else if (plcComConfig.Channel is EthernetListener)
            {
                ListenerView listenerView = new ListenerView(plcComConfig);
                listenerView.ForcePlcName(m_ForcePlcName);
                panel1.Controls.Clear();
                panel1.Controls.Add(listenerView);
                listenerView.OnSetAsDirty += new EventHandler(OnSetAsDirty);
                listenerView.OnCheckConnection += new EventHandler(OnCheckConnection);
                listenerView.Dock = DockStyle.Fill;
            }
            else
            {
                System.Diagnostics.Debug.Assert(false);
            }
        }
        
     
        private void loadChannelsComboFromList(List<Channels> channels)
        {
            if (channels.Count == 0)
                throw new InvalidEnumArgumentException("You must supply at least 1 channel");

            cmbConnectionType.BeginUpdate();
            cmbConnectionType.Items.Clear();
            foreach (Channels channel in channels)
            {
                PlcComConfig plcComConfig = null;
                switch (channel)
                {
                    case Channels.Serial:
                        plcComConfig = new PlcComConfig(Presenters.SerialPresenter.GetDefaultChannel());
                        break;
                    case Channels.Ethernet:
                        plcComConfig = new PlcComConfig(Presenters.EthernetPresenter.GetDefaultChannel());
                        break;
                    case Channels.EthernetListener:
                        plcComConfig = new PlcComConfig(Presenters.ListenerPresenter.GetDefaultChannel());
                        break;
                }

                cmbConnectionType.Items.Add(plcComConfig);
            }
            cmbConnectionType.EndUpdate();
            cmbConnectionType.SelectedIndex = -1;
            cmbConnectionType.SelectedIndex = 0;
        }

        private void cmbConnectionType_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (cmbConnectionType.SelectedIndex == -1)
                return;

            PlcComConfig plcComConfig = (PlcComConfig)cmbConnectionType.SelectedItem;
            plcInformation1.Presenter.PlcComConfig = plcComConfig;
            favoritesView1.PlcComConfig = plcComConfig;
            setChannelPresenterByPlcComConfig(plcComConfig);

            DirtyBit = true;
        }

        void favoritesView1_OnFavoriteRowLoad(object sender, EventArgs e)
        {
            PlcComConfig plcComConfig = (PlcComConfig)sender;
            for (int i = 0; i < cmbConnectionType.Items.Count; i++)
            {
                Channel channel = ((PlcComConfig)cmbConnectionType.Items[i]).Channel;
                if (channel.GetType() == plcComConfig.Channel.GetType())
                {
                    cmbConnectionType.Items[i] = plcComConfig;
                    //setChannelPresenterByPlcComConfig(plcComConfig);
                    cmbConnectionType.SelectedIndex = -1;
                    cmbConnectionType.SelectedIndex = i;
                    explorerBarMain.Groups[0].Select();
                    DirtyBit = true;
                    return;
                }
            }

            throw new ApplicationException("The communication Channel in the Favorites file is not one of the options of the Connection Types");

        }

        void OnCheckConnection(object sender, EventArgs e)
        {
            inCheckConnectionMode = true;
            plcInformation1.GetPlcInformation();
        }

        void plcInformation1_OnGetPlcInformationStart(object sender, EventArgs e)
        {
            IChannelView channelView = (panel1.Controls[0] as IChannelView);
            cmbConnectionType.Enabled = false;
            channelView.SetCheckConnectionEnableState(false);
        }

        void plcInformation1_OnGetPlcInformationFinished(bool success)
        {
            IChannelView channelView = (panel1.Controls[0] as IChannelView);
            cmbConnectionType.Enabled = true;
            channelView.SetCheckConnectionEnableState(true);
            if (inCheckConnectionMode && success)
            {
                explorerBarMain.Groups[1].Select();
            }

            inCheckConnectionMode = false;
        }

        void OnSetAsDirty(object sender, EventArgs e)
        {
            DirtyBit = true;
        }
    }
}

internal class resfinder { }