﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Unitronics.ComConfig.Lite.Presenters;
using Unitronics.ComConfig.Lite.Globals;
using Unitronics.ComDriver;
using System.Net;

namespace Unitronics.ComConfig.Lite
{
    internal partial class EthernetView : UserControl, IEthernetView, IChannelView
    {
        public event EventHandler OnSetAsDirty;
        public event EventHandler OnCheckConnection;
        private Presenters.EthernetPresenter m_Presenter;
        private bool m_IgnoreEvents;
        private bool hostNameActive = false;
        private bool ipHostFieldReopening = false;

        public EthernetView() {
            InitializeComponent();
            ipTarget.Width = cmbRetries.Width;
            m_Presenter = new EthernetPresenter(this);
            m_Presenter.OnSetAsDirty += new EventHandler(m_Presenter_OnSetAsDirty);
            prvInitGUI();
            SetSettingsToGUI();
            toolTip1.SetToolTip(btnToggleIPHostName, "Toggle Target IP / Host Name");
        }

        public EthernetView(PlcComConfig plcComConfig) {
            InitializeComponent();
            ipTarget.Width = cmbRetries.Width;
            m_Presenter = new EthernetPresenter(this, plcComConfig);
            m_Presenter.OnSetAsDirty += new EventHandler(m_Presenter_OnSetAsDirty);
            prvInitGUI();
            SetSettingsToGUI();
            toolTip1.SetToolTip(btnToggleIPHostName, "Toggle Target IP / Host Name");
        }

        #region iEthernetView
        public event ValueChangedEventHandler<string> IpAddressChanged;
        public event ValueChangedEventHandler<int> EthRemotePortChanged;
        public event ValueChangedEventHandler<EthProtocol> EthernetProtocolChanged;
        public event CancelEventHandler IpAddressValidating;
        public event ValueChangedEventHandler<int> TimeoutChanged;
        public event ValueChangedEventHandler<int> RetriesChanged;
        public event ValueChangedEventHandler<int> UnitIdChanged;
        public event ValueChangedEventHandler<string> HostNameChanged;
        public event EventHandler Loading;

        public void SetSettingsToGUI() {

            m_IgnoreEvents = true;

            txtTargetPort.Text = (m_Presenter.PlcComConfig.Channel as Ethernet).RemotePort.ToString();

            IPAddress ipAddress;
            string ipAddressOrHostName = (m_Presenter.PlcComConfig.Channel as Ethernet).RemoteIP.ToString();
            if (IPAddress.TryParse(ipAddressOrHostName, out ipAddress))
            {
                this.ipTarget.Text = ipAddressOrHostName;
                txtHostName.Enabled = false;
            }
            else
            {
                ipTarget.Enabled = false;
                this.ipTarget.Text = "192.168.0.1";
                lblTargetIP.Text = "Host:";
                this.txtHostName.Text = ipAddressOrHostName;
                hostNameActive = true;
                this.ipTarget.Left = -this.ipTarget.Width;
                this.txtHostName.Left = 0;
            }
            
            Services.SetComboValue<EthProtocol>((m_Presenter.PlcComConfig.Channel as Ethernet).Protocol, cmbProtocol);
            Services.SetComboValue<int>((m_Presenter.PlcComConfig.Channel as Ethernet).TimeOut, cmbTimeout);
            Services.SetComboValue<int>((m_Presenter.PlcComConfig.Channel as Ethernet).Retry, cmbRetries);
            Services.SetComboValue<int>(m_Presenter.PlcComConfig.UnitID, cmbUnitID);

            fillPlcName();
            fillPlcNameCheckbox();

            m_IgnoreEvents = false;
        }
   

        #endregion

        private void prvInitGUI() {

            Services.FillEthProtocolCombo(cmbProtocol);
            Services.FillTimeoutCombo(cmbTimeout);
            Services.FillRetriesCombo(cmbRetries);
            Services.FillUnitIDCombo(cmbUnitID);

            lblPlcName.Enabled = chkRequirePlcName.Checked;
            txtPlcName.Enabled = chkRequirePlcName.Checked;
        }        

        private void fillPlcName() {

            if (m_IgnoreEvents)
                return;

            txtPlcName.Text = m_Presenter.PlcComConfig.PlcName;
        }

        private void fillPlcNameCheckbox() {
            
            if (m_IgnoreEvents)
                return;

            chkRequirePlcName.Checked = m_Presenter.PlcComConfig.RequirePlcName;
        }

        private void cmbRetries_SelectedIndexChanged(object sender, EventArgs e) {

            if(m_IgnoreEvents)
                return;

            ComboItem<int> item = (cmbRetries.SelectedItem as ComboItem<int>);
            ValueChangedEventArgs<int> ea = new ValueChangedEventArgs<int>(item.Value);
            prvRetriesChangedEvent(sender, ea);
        }

        private void cmbTimeout_SelectedIndexChanged(object sender, EventArgs e) {

            if(m_IgnoreEvents)
                return;

            ComboItem<int> item = (cmbTimeout.SelectedItem as ComboItem<int>);
            ValueChangedEventArgs<int> ea = new ValueChangedEventArgs<int>(item.Value);
            prvRaiseTimeoutChangedEvent(sender, ea);
        }

        private void prvRaiseTimeoutChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if (m_IgnoreEvents)
                return;

            if(null != TimeoutChanged) {
                TimeoutChanged(sender, e);
            }
        }

        private void prvRetriesChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if (m_IgnoreEvents)
                return;

            if(null != RetriesChanged) {
                RetriesChanged(sender, e);
            }
        }

       private void prvRaiseIpChangedEvent(object sender, ValueChangedEventArgs<string> e) {

           if (m_IgnoreEvents)
               return;

            if(null != IpAddressChanged) {
                IpAddressChanged(sender, e);
            }
        }

        private void prvRaiseIpAddressValidatingEvent(object sender, CancelEventArgs e) {
            if(null != IpAddressValidating) {
                IpAddressValidating(sender, e);
            }
        }

        private void prvEthRemotePortChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if (m_IgnoreEvents)
                return;

            if(null != EthRemotePortChanged) {
                EthRemotePortChanged(sender, e);
            }
        }

        private void prvEthProtocolChangedEvent(object sender, ValueChangedEventArgs<EthProtocol> e) {

            if (m_IgnoreEvents)
                return;

            if(null != EthernetProtocolChanged) {
                EthernetProtocolChanged(sender, e);
            }
        }

        private void cmbUnitID_SelectedIndexChanged(object sender, EventArgs e) {
            if(m_IgnoreEvents)
                return;
            ComboItem<int> item = (cmbUnitID.SelectedItem as ComboItem<int>);
            ValueChangedEventArgs<int> ea = new ValueChangedEventArgs<int>(item.Value);
            prvRaiseUnitIdChangedEvent(sender, ea);
        }

        private void prvRaiseUnitIdChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if(null != UnitIdChanged) {
                UnitIdChanged(sender, e);
            }
        }

        private void txtEthPort_Validating(object sender, CancelEventArgs e) {

            if(m_IgnoreEvents) {
                return;
            }
            UInt16 iNewPort;
            if (!UInt16.TryParse(((TextBox)sender).Text, out iNewPort) || iNewPort == 0)
            {
                ((TextBox)sender).Text = (m_Presenter.PlcComConfig.Channel as Ethernet).RemotePort.ToString();
                //e.Cancel = true;
            }
        }

        private void txtEthPort_Validated(object sender, EventArgs e) {

            if(m_IgnoreEvents) {
                return;
            }
            UInt16 iNewPort = UInt16.Parse(((TextBox)sender).Text);
            prvRaiseEthRemotePortChangedEvent(new ValueChangedEventArgs<int>(iNewPort));
        }

        private void prvRaiseEthRemotePortChangedEvent(ValueChangedEventArgs<int> e) {

            if(null != EthRemotePortChanged) {
                EthRemotePortChanged(this, e);
            }
        }

        private void cboEthProtocol_SelectedIndexChanged(object sender, EventArgs e) {

            ComboItem<EthProtocol> item = (cmbProtocol.SelectedItem as ComboItem<EthProtocol>);
            ValueChangedEventArgs<EthProtocol> ea = new ValueChangedEventArgs<EthProtocol>(item.Value);
            prvRaiseEthProtocolChangedEvent(ea);
        }

        private void prvRaiseEthProtocolChangedEvent(ValueChangedEventArgs<EthProtocol> e) {

            if(null != EthernetProtocolChanged) {
                EthernetProtocolChanged(this, e);
            }
        }

        private void prvRaiseIpAddressChangedEvent(ValueChangedEventArgs<string> e) {

            if(null != IpAddressChanged) {
                IpAddressChanged(this, e);
            }
        }

        private void ipAddrEditor_Validated(object sender, EventArgs e) {

            errBlink.Clear();
            prvRaiseIpAddressChangedEvent(new ValueChangedEventArgs<string>(ipTarget.Text));

        }


        private void ipAddrEditor_Validating(object sender, CancelEventArgs e) {

            if(null != IpAddressValidating) {
                IpAddressValidating(sender, e);
            }

        }

        //private void EthernetView_Load(object sender, EventArgs e) {

        //    if(null != Loading) {
        //        Loading(sender, e);
        //    }
        //}

        private void txtPlcName_TextChanged(object sender, EventArgs e)
        {
            if (m_IgnoreEvents)
                return;

            m_Presenter.PlcComConfig.PlcName = txtPlcName.Text;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        private void chkRequirePlcName_CheckedChanged(object sender, EventArgs e)
        {
            if (m_IgnoreEvents)
                return;

            m_Presenter.PlcComConfig.RequirePlcName = chkRequirePlcName.Checked;

            lblPlcName.Enabled = chkRequirePlcName.Checked;
            txtPlcName.Enabled = chkRequirePlcName.Checked;

            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        void m_Presenter_OnSetAsDirty(object sender, EventArgs e)
        {
            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }

        #region IChannelView Members

        public PlcComConfig GetPlcComConfig()
        {
            return m_Presenter.PlcComConfig;
        }

        public void ForcePlcName(bool value)
        {
            if (value)
            {
                chkRequirePlcName.Checked = true;
                chkRequirePlcName.Enabled = false;
            }
            else
            {
                chkRequirePlcName.Enabled = true;
            }
        }

        public void SetCheckConnectionEnableState(bool enable)
        {
            btnCheckConnection.Enabled = enable;
        }

        #endregion

        private void btnCheckConnection_Click(object sender, EventArgs e)
        {
            if (OnCheckConnection != null)
            {
                OnCheckConnection(null, null);
            }
        }

        private void btnToggleIPHostName_Click(object sender, EventArgs e)
        {
            hostNameActive = !hostNameActive;
            if (hostNameActive)
            {
                txtHostName.Enabled = true;
                txtHostName.Focus();
            }
            else
            {
                ipTarget.Enabled = true;
                ipTarget.Focus();
            }
            btnToggleIPHostName.Enabled = false;
            ipHostFieldReopening = false;
            timerAnimation.Start();

        }

        private void timerAnimation_Tick(object sender, EventArgs e)
        {
            if (!ipHostFieldReopening)
            {
                if (hostNameActive)
                {
                    ipTarget.Left -= 10;
                    if (ipTarget.Right <= 0)
                    {
                        ipTarget.Enabled = false;
                        ipTarget.Left = -ipTarget.Width;
                        ipHostFieldReopening = true;
                        lblTargetIP.Text = "Host:";
                    }
                }
                else
                {
                    txtHostName.Left -= 10;
                    if (txtHostName.Right <= 0)
                    {
                        txtHostName.Enabled = false;
                        txtHostName.Left = -txtHostName.Width;
                        ipHostFieldReopening = true;
                        lblTargetIP.Text = "Target IP:";   
                    }
                }
            }
            else
            {
                if (hostNameActive)
                {
                    txtHostName.Left += 10;
                    if (txtHostName.Left >= 0)
                    {
                        txtHostName.Left = 0;
                        ipHostFieldReopening = false;
                        timerAnimation.Stop();
                        btnToggleIPHostName.Enabled = true;
                    }
                }
                else
                {
                    ipTarget.Left += 10;
                    if (ipTarget.Left >= 0)
                    {
                        ipTarget.Left = 0;
                        ipHostFieldReopening = false;
                        timerAnimation.Stop();
                        btnToggleIPHostName.Enabled = true;
                    }
                }
            }
        }

        private void txtHostName_Validated(object sender, EventArgs e)
        {
            if (m_IgnoreEvents)
                return;

            if (HostNameChanged != null)
                HostNameChanged(this, new ValueChangedEventArgs<string>(txtHostName.Text));
        }

        private void txtHostName_Validating(object sender, CancelEventArgs e)
        {
            return;
        }

    }
}
