﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.ComponentModel.Design;

namespace ExplorerBarLite
{
    [Designer(typeof(CollectionControlDesigner))]
    [TypeConverter(typeof(ExplorerBar)), DesignTimeVisible(false), ToolboxItem(false)]
    //[Designer("System.Windows.Forms.Design.ParentControlDesigner, System.Design", typeof(IDesigner))]
    public partial class ExplorerBar : UserControl
    {
        private GroupsCollection groups;
        private ExplorerBarGroup selectedGroup = null;
        private ExplorerBarGroup newlySelectedGroup = null;
        private int animationCounter = 0;

        public ExplorerBar()
        {
            InitializeComponent();
            groups = new GroupsCollection(this);
        }

        [Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public GroupsCollection Groups
        {
            get
            {
                return groups;
            }
        }

        internal void OnSelectionChanged()
        {
            ExplorerBarGroup group = null;
            ISelectionService s = (ISelectionService)GetService(typeof(ISelectionService));

            if (s.PrimarySelection is ExplorerBarGroup)
            {
                group = s.PrimarySelection as ExplorerBarGroup;
                if (group.IsSelected == false)
                    group.Select();
            }
            // See if the primary selection is one of our buttons
            //foreach (ColourButton button in Buttons)
            //{
            //    if (s.PrimarySelection == button)
            //    {
            //        newHighlightedButton = button;
            //        break;
            //    }
            //}

            //// Apply if necessary
            //if (newHighlightedButton != highlightedButton)
            //{
            //    highlightedButton = newHighlightedButton;
            //    Invalidate();
            //}
        }

        internal void CalculateLayout()
        {
            this.SuspendLayout();
            this.Controls.Clear();
            bool selectionFound = false;
            int selectedHeight = this.Height - 22 * (Groups.Count - 1) - 2; // 2 more pixels for the fixed single boarder
            for (int i = Groups.Count - 1; i >= 0; i--)
            {
                if (Groups[i].IsSelected)
                {
                    if (!selectionFound)
                    {
                        selectionFound = true;
                        selectedGroup = Groups[i];
                    }
                    else
                    {
                        Groups[i].IsSelected = false;
                    }
                    
                }
            }
            if (!selectionFound && Groups.Count > 0)
            {
                Groups[0].IsSelected = true;
            }
            for (int i = Groups.Count - 1; i >= 0; i--)
            {
                Groups[i].OnNewGroupSelection -= new EventHandler(ExplorerBar_OnNewGroupSelection);
                Groups[i].Height = 22;
                this.Controls.Add(Groups[i]);
                Groups[i].Dock = DockStyle.Top;
                if (Groups[i].IsSelected)
                {
                    Groups[i].Height = selectedHeight;
                }
                else
                {
                    Groups[i].Height = 22;
                }
                Groups[i].OnNewGroupSelection += new EventHandler(ExplorerBar_OnNewGroupSelection);
            }

            //foreach (ExplorerBarGroup group in Groups)
            //{
            //    System.Diagnostics.Debug.Print(group.Name);
            //    this.Controls.Add(group);
            //    group.Dock = DockStyle.Top;
            //}
            this.ResumeLayout();
        }

        void ExplorerBar_OnNewGroupSelection(object sender, EventArgs e)
        {
            if (newlySelectedGroup != null)
                return;

            newlySelectedGroup = sender as ExplorerBarGroup;
            selectedGroup.Container.Visible = false;
            newlySelectedGroup.Container.Visible = false;
            animationCounter = 0;
            timer1.Enabled = true;
            timer1.Start();
        }

        private void ExplorerBar_Resize(object sender, EventArgs e)
        {
            CalculateLayout();
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if ((selectedGroup == null) || (newlySelectedGroup == null))
                return;

            int stepSize = (this.Height - 22 * (groups.Count - 1) - 2) / 10; // 2 more pixels for the fixed single boarder
            if (animationCounter < 10)
            {
                if ((selectedGroup.Height >= 22 + stepSize))
                {
                    this.SuspendLayout();
                    selectedGroup.Height -= stepSize;
                    newlySelectedGroup.Height += stepSize;
                    this.ResumeLayout();
                }
                else
                {
                    int lastStep = selectedGroup.Height - 22;
                    selectedGroup.Height = 22;
                    newlySelectedGroup.Height += lastStep;
                    animationFinished();
                }
            }
            else
            {
                animationFinished();
            }
        }

        private void animationFinished()
        {
            this.SuspendLayout();
            selectedGroup.Container.Visible = true;
            newlySelectedGroup.Container.Visible = true;
            newlySelectedGroup.IsSelected = true;
            selectedGroup.IsSelected = false;
            selectedGroup = newlySelectedGroup;
            newlySelectedGroup = null;
            timer1.Stop();
            timer1.Enabled = false;
            this.ResumeLayout();
        }

        private void ExplorerBar_Load(object sender, EventArgs e)
        {
            CalculateLayout();
        }
    }

    internal class CollectionControlDesigner : ControlDesigner
    {
        private ExplorerBar MyControl;

        public override void Initialize(IComponent component)
        {
            base.Initialize(component);

            // Record instance of control we're designing
            MyControl = (ExplorerBar)component;

            // Hook up events
            ISelectionService s = (ISelectionService)GetService(typeof(ISelectionService));
            IComponentChangeService c = (IComponentChangeService)GetService(typeof(IComponentChangeService));
            s.SelectionChanged += new EventHandler(OnSelectionChanged);
            c.ComponentRemoving += new ComponentEventHandler(OnComponentRemoving);

        }

        private void OnSelectionChanged(object sender, System.EventArgs e)
        {
            MyControl.OnSelectionChanged();
            //if (sender is ExplorerBarGroup)
            //{
            //    ExplorerBarGroup group = sender as ExplorerBarGroup;
            //    if (group.IsSelected == false)
            //        group.Select();
            //}
            //MessageBox.Show("Selection Changed");
            //MyControl.OnSelectionChanged();
        }

        private void OnComponentRemoving(object sender, ComponentEventArgs e)
        {
            //MessageBox.Show("Removing");
            //IComponentChangeService c = (IComponentChangeService)GetService(typeof(IComponentChangeService));
            //ColourButton button;
            //IDesignerHost h = (IDesignerHost)GetService(typeof(IDesignerHost));
            //int i;

            //// If the user is removing a button
            //if (e.Component is ColourButton)
            //{
            //    button = (ColourButton)e.Component;
            //    if (MyControl.Buttons.Contains(button))
            //    {
            //        c.OnComponentChanging(MyControl, null);
            //        MyControl.Buttons.Remove(button);
            //        c.OnComponentChanged(MyControl, null, null, null);
            //        return;
            //    }
            //}

            //// If the user is removing the control itself
            //if (e.Component == MyControl)
            //{
            //    for (i = MyControl.Buttons.Count - 1; i >= 0; i--)
            //    {
            //        button = MyControl.Buttons[i];
            //        c.OnComponentChanging(MyControl, null);
            //        MyControl.Buttons.Remove(button);
            //        h.DestroyComponent(button);
            //        c.OnComponentChanged(MyControl, null, null, null);
            //    }
            //}
        }


        protected override void Dispose(bool disposing)
        {
            ISelectionService s = (ISelectionService)GetService(typeof(ISelectionService));
            IComponentChangeService c = (IComponentChangeService)GetService(typeof(IComponentChangeService));
            s.SelectionChanged -= new EventHandler(OnSelectionChanged);
            c.ComponentRemoving -= new ComponentEventHandler(OnComponentRemoving);
            base.Dispose(disposing);
        }

        //public override System.Collections.ICollection AssociatedComponents
        //{
        //    get
        //    {
        //        return MyControl.Groups;
        //    }
        //}

        public override System.ComponentModel.Design.DesignerVerbCollection Verbs
        {
            get
            {
                DesignerVerbCollection v = new DesignerVerbCollection();

                // Verb to add buttons
                v.Add(new DesignerVerb("&Add Group", new EventHandler(OnAddGroup)));

                return v;
            }
        }

        private void OnAddGroup(object sender, System.EventArgs e)
        {
            ExplorerBarGroup group;
            Panel panel;
            IDesignerHost h = (IDesignerHost)GetService(typeof(IDesignerHost));
            DesignerTransaction dt;

            IComponentChangeService c = (IComponentChangeService)GetService(typeof(IComponentChangeService));

            // Add a new button to the collection
            dt = h.CreateTransaction("Add Group");
            group = (ExplorerBarGroup)h.CreateComponent(typeof(ExplorerBarGroup));
            panel = (Panel)h.CreateComponent(typeof(Panel));
            //ExplorerBarGroup group = new ExplorerBarGroup();
            group.Key = "New Group";
            c.OnComponentChanging(MyControl, null);
            MyControl.Groups.Add(group);
            group.ExplorerBar = MyControl;
            group.Container = panel;
            c.OnComponentChanged(MyControl, null, null, null);
            dt.Commit();
        }

        //protected override bool GetHitTest(System.Drawing.Point point)
        //{
        //    Rectangle wrct;

        //    point = MyControl.PointToClient(point);

        //    foreach (ColourButton button in MyControl.Buttons)
        //    {
        //        wrct = button.Bounds;
        //        if (wrct.Contains(point))
        //            return true;
        //    }

        //    return false;
        //}

    }
}
