﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Unitronics.ComConfig.Lite.Presenters;
using Unitronics.ComDriver;

namespace Unitronics.ComConfig.Lite
{
    internal partial class PlcInformation : UserControl
    {
        public delegate void GetPlcInformationFinishedDelegate(bool success);
        public event EventHandler OnGetPlcInformationStart;
        public event GetPlcInformationFinishedDelegate OnGetPlcInformationFinished;

        private Presenters.PlcInformationPresenter m_Presenter;
        public PlcInformation()
        {
            InitializeComponent();
            m_Presenter = new PlcInformationPresenter();
            m_Presenter.OnGetPlcInformation += new EventHandler(m_Presenter_OnGetPlcInformation);
            m_Presenter.OnGuiStateChanged += new EventHandler(m_Presenter_OnGuiStateChanged);
            m_Presenter.OnGetPlcInformationStart += new EventHandler(m_Presenter_OnGetPlcInformationStart);
            m_Presenter.OnGetPlcInformationFinished += new GetPlcInformationFinishedDelegate(m_Presenter_OnGetPlcInformationFinished);
        }

        public void GetPlcInformation()
        {
            getPlcInformation();
        }

        void m_Presenter_OnGuiStateChanged(object sender, EventArgs e)
        {
            Presenters.PlcInformationPresenter.GuiState guiState = sender as Presenters.PlcInformationPresenter.GuiState;
            MethodInvoker uiDelegate = delegate
            {
                btnGetInformation.Enabled = guiState.GetPlcInformationButtonEnabled;
                BusyCursor.Visible = guiState.BusyCursorVisible;
            };
            UpdateUI(uiDelegate);
        }

        protected void UpdateUI(MethodInvoker uiDelegate)
        {
            if (InvokeRequired)
                this.Invoke(uiDelegate);
            else
                uiDelegate();
        }

        void m_Presenter_OnGetPlcInformation(object sender, EventArgs e)
        {
            PlcVersion version = (PlcVersion)((object[])sender)[0];
            string plcName = (string)((object[])sender)[1];

            MethodInvoker uiDelegate = delegate
            {
                txtModel.Text = version.OPLCModel;
                txtHW.Text = version.HWVersion;
                txtOS.Text = version.OSVersion + " (" + version.OSBuildNum + ")";
                txtBoot.Text = version.Boot;
                txtBinLib.Text = version.BinLib;
                txtFactoryBoot.Text = version.FactoryBoot;
                txtPlcName.Text = plcName;

            };
            UpdateUI(uiDelegate);
        }
        internal PlcInformationPresenter Presenter
        {
            get
            {
                return m_Presenter;
            }
        }

        private void btnGetInformation_Click(object sender, EventArgs e)
        {
            getPlcInformation(); 
        }

        private void getPlcInformation()
        {
            txtModel.Text = "";
            txtHW.Text = "";
            txtOS.Text = "";
            txtBoot.Text = "";
            txtBinLib.Text = "";
            txtFactoryBoot.Text = "";
            txtPlcName.Text = "";

            m_Presenter.GetPlcInformation();
        }

        void m_Presenter_OnGetPlcInformationStart(object sender, EventArgs e)
        {
            MethodInvoker uiDelegate = delegate
            {
                if (OnGetPlcInformationStart != null)
                    OnGetPlcInformationStart(sender, e);
            };
            UpdateUI(uiDelegate);  
        }

        void m_Presenter_OnGetPlcInformationFinished(bool success)
        {
            MethodInvoker uiDelegate = delegate
            {
                if (OnGetPlcInformationFinished != null)
                    OnGetPlcInformationFinished(success);
            };
            UpdateUI(uiDelegate);  
        }

        private void PlcInformation_Resize(object sender, EventArgs e)
        {
            this.BusyCursor.Left = (this.Width - this.BusyCursor.Width) / 2;
            this.BusyCursor.Top = (this.Height - this.BusyCursor.Height) / 2;
        }
    }
}
