﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.ComponentModel;
using Unitronics.ComDriver;
using Unitronics.ComConfig.Lite.Globals;

namespace Unitronics.ComConfig.Lite.Presenters
{
    class EthernetPresenter
    {
        private Ethernet m_Ethernet;
        private PlcComConfig m_PlcComConfig;
        private IEthernetView m_View;
        public event EventHandler OnSetAsDirty;

        public EthernetPresenter(IEthernetView pView)
        {
            m_View = pView;
            prvInitView();
            m_PlcComConfig = new PlcComConfig();
            m_Ethernet = (Ethernet)GetDefaultChannel();
            m_PlcComConfig.Channel = m_Ethernet;
        }

        public EthernetPresenter(IEthernetView pView,PlcComConfig plcComConfig)
        {
            m_View = pView;
            prvInitView();
            m_PlcComConfig = plcComConfig;
            m_Ethernet = (Ethernet)plcComConfig.Channel;
        }

        private void prvInitView() {

            m_View.RetriesChanged += new ValueChangedEventHandler<int>(OnRetriesChanged);
            m_View.TimeoutChanged += new ValueChangedEventHandler<int>(OnTimeoutChanged);
            m_View.IpAddressChanged += new ValueChangedEventHandler<string>(OnIpChangedEvent);
            m_View.EthRemotePortChanged += new ValueChangedEventHandler<int>(OnEthRemotePortChangedEvent);
            m_View.EthernetProtocolChanged += new ValueChangedEventHandler<EthProtocol>(OnEthProtocolChangedEvent);
            m_View.IpAddressValidating += new System.ComponentModel.CancelEventHandler(OnIpAddressValidating);
            m_View.HostNameChanged += new ValueChangedEventHandler<string>(OnHostNameChangedEvent);
            m_View.Loading += new EventHandler(Loading);
            m_View.UnitIdChanged += new ValueChangedEventHandler<int>(OnUnitIdChanged);
        }

        public PlcComConfig PlcComConfig
        {
            get
            {
                return m_PlcComConfig;
            }
            set
            {
                m_PlcComConfig = value;
            }
        }

        public static Channel GetDefaultChannel()
        {
            Ethernet ethernet = new Ethernet("192.168.0.1", 20256,  EthProtocol.TCP, 3, 1000);
            return ethernet;
        }

        private void OnRetriesChanged(object sender, ValueChangedEventArgs<int> e) {

            if (m_PlcComConfig.Channel.Retry != e.Value)
                SetAsDirty();

            m_PlcComConfig.Channel.Retry = e.Value;
            m_View.SetSettingsToGUI();
        }
        private void OnTimeoutChanged(object sender, ValueChangedEventArgs<int> e) {

            if (((Ethernet)m_PlcComConfig.Channel).TimeOut != e.Value)
                SetAsDirty();

            ((Ethernet)m_PlcComConfig.Channel).TimeOut = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void OnUnitIdChanged(object sender, ValueChangedEventArgs<int> e) {

            if (m_PlcComConfig.UnitID != e.Value)
                SetAsDirty();

            m_PlcComConfig.UnitID = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void Loading(object sender, EventArgs e) {

            m_View.SetSettingsToGUI();
        }

        private void OnIpChangedEvent(object sender, ValueChangedEventArgs<string> e) {

            if (((Ethernet)m_PlcComConfig.Channel).RemoteIP != e.Value)
                SetAsDirty();

            Ethernet oldEthernet = (Ethernet)m_PlcComConfig.Channel;
            m_PlcComConfig.Channel = new Ethernet(e.Value, oldEthernet.RemotePort, oldEthernet.Protocol, oldEthernet.Retry, oldEthernet.TimeOut);
            //((Ethernet)m_PlcComConfig.Channel).RemoteIP = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void OnHostNameChangedEvent(object sender, ValueChangedEventArgs<string> e)
        {

            if (((Ethernet)m_PlcComConfig.Channel).RemoteIP != e.Value)
                SetAsDirty();

            Ethernet oldEthernet = (Ethernet)m_PlcComConfig.Channel;
            m_PlcComConfig.Channel = new Ethernet(e.Value, oldEthernet.RemotePort, oldEthernet.Protocol, oldEthernet.Retry, oldEthernet.TimeOut);
            //((Ethernet)m_PlcComConfig.Channel).RemoteIP = e.Value;
            m_View.SetSettingsToGUI();
        }
  
        private void OnEthRemotePortChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if (((Ethernet)m_PlcComConfig.Channel).RemotePort != e.Value)
                SetAsDirty();

            Ethernet oldEthernet = (Ethernet)m_PlcComConfig.Channel;
            m_PlcComConfig.Channel = new Ethernet(oldEthernet.RemoteIP, e.Value, oldEthernet.Protocol, oldEthernet.Retry, oldEthernet.TimeOut);
            //((Ethernet)m_PlcComConfig.Channel).RemotePort = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void OnEthProtocolChangedEvent(object sender, ValueChangedEventArgs<EthProtocol> e) {

            if (((Ethernet)m_PlcComConfig.Channel).Protocol != e.Value)
                SetAsDirty();

            ((Ethernet)m_PlcComConfig.Channel).Protocol = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void OnIpAddressValidating(object sender, CancelEventArgs e) {

            try {
                prvGetIpAddressFromText((sender as Unitronics.ComConfig.Lite.IPAddressControl).Text);
            } catch {
                e.Cancel = true;
                m_View.SetSettingsToGUI();
            }
        }

        private IPAddress prvGetIpAddressFromText(string pIpString) {

            string[] ipFields = pIpString.Split(new string[] { "." }, StringSplitOptions.None);
            byte[] ipBytes = new byte[ipFields.Length];
            int iByteIndex = 0;
            foreach(string ipField in ipFields) {
                ipBytes[iByteIndex++] = byte.Parse(ipField);
            }
            IPAddress ip = new IPAddress(ipBytes);
            return ip;
        }

        private void SetAsDirty()
        {
            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }
    }
}
