﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Unitronics.ComDriver;
using Unitronics.ComConfig.Lite.Presenters;
using Unitronics.ComConfig.Lite.Globals;

namespace Unitronics.ComConfig.Lite.Presenters {
    class ListenerPresenter {
        private EthernetListener m_EthernetListener;
        private PlcComConfig m_PlcComConfig;
        private IListenerView m_View;
        public event EventHandler OnSetAsDirty;

        public ListenerPresenter(IListenerView pView) {
            m_View = pView;
            prvInitView();
            m_PlcComConfig = new PlcComConfig();
            m_EthernetListener = (EthernetListener)GetDefaultChannel();
            m_PlcComConfig.Channel = m_EthernetListener;
        }

        public ListenerPresenter(IListenerView pView, PlcComConfig plcComConfig) {
            m_View = pView;
            prvInitView();
            m_PlcComConfig = plcComConfig;
            m_EthernetListener = (EthernetListener)plcComConfig.Channel;
        }

        public PlcComConfig PlcComConfig {
            get {
                return m_PlcComConfig;
            }
            set {
                m_PlcComConfig = value;
            }
        }

        public static Channel GetDefaultChannel() {
            EthernetListener listener = new EthernetListener();
            listener.TimeOut = 1000;
            listener.Retry = 3;
            return listener;
        }

        private void prvInitView() {

            m_View.RetriesChanged += new ValueChangedEventHandler<int>(OnRetriesChanged);
            m_View.TimeoutChanged += new ValueChangedEventHandler<int>(OnTimeoutChanged);
            m_View.EthLocalPortChanged += new ValueChangedEventHandler<int>(OnEthPortChangedEvent);
            m_View.Loading += new EventHandler(Loading);
        }

        private void Loading(object sender, EventArgs e) {

            m_View.SetSettingsToGUI();
        }

        private void OnEthPortChangedEvent(object sender, ValueChangedEventArgs<int> e) {

            if (((EthernetListener)m_PlcComConfig.Channel).LocalPort != e.Value)
                SetAsDirty();

            EthernetListener oldListener = (EthernetListener)m_PlcComConfig.Channel;
            m_PlcComConfig.Channel = new EthernetListener(e.Value, oldListener.Retry, oldListener.TimeOut);
            //((EthernetListener)m_PlcComConfig.Channel).LocalPort = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void OnTimeoutChanged(object sender, ValueChangedEventArgs<int> e) {

            if (m_PlcComConfig.Channel.TimeOut != e.Value)
                SetAsDirty();

            ((EthernetListener)m_PlcComConfig.Channel).TimeOut = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void OnRetriesChanged(object sender, ValueChangedEventArgs<int> e) {

            if (m_PlcComConfig.Channel.Retry != e.Value)
                SetAsDirty();

            m_PlcComConfig.Channel.Retry = e.Value;
            m_View.SetSettingsToGUI();
        }

        private void SetAsDirty()
        {
            if (OnSetAsDirty != null)
                OnSetAsDirty(null, null);
        }
    }
}
