﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Unitronics.ComDriver;
using System.Windows.Forms;

namespace Unitronics.ComConfig.Lite.Presenters
{
    class PlcInformationPresenter
    {
        public event EventHandler OnGetPlcInformationStart;
        public event PlcInformation.GetPlcInformationFinishedDelegate OnGetPlcInformationFinished;

        internal class GuiState
        {
            public bool GetPlcInformationButtonEnabled;
            public bool BusyCursorVisible;
        }

        public PlcComConfig PlcComConfig { get; set; }
        internal event EventHandler OnGetPlcInformation;
        internal event EventHandler OnGuiStateChanged;

        private GuiState m_GuiState;

        public PlcInformationPresenter()
        {
            m_GuiState = new GuiState()
            {
                BusyCursorVisible = false,
                GetPlcInformationButtonEnabled = true,
            };
        }

        public void GetPlcInformation()
        {
            m_GuiState.GetPlcInformationButtonEnabled = false;
            m_GuiState.BusyCursorVisible = true;

            if (OnGetPlcInformationStart != null)
                OnGetPlcInformationStart(null, null);

            if (OnGuiStateChanged != null)
                OnGuiStateChanged(m_GuiState, null);

            MethodInvoker methodInvoker = new MethodInvoker(getPlcInformation);
            methodInvoker.BeginInvoke(null, null);
        }

        private void getPlcInformation()
        {
            try
            {
                Channel channel = null;

                if (PlcComConfig.Channel is Serial)
                {
                    channel = PLCFactory.GetChannel((PlcComConfig.Channel as Serial).PortName);
                }
                else if (PlcComConfig.Channel is Ethernet)
                {
                    Ethernet ethernet = PlcComConfig.Channel as Ethernet;
                    channel = PLCFactory.GetChannel(ethernet.RemoteIP, ethernet.RemotePort);
                }
                else if (PlcComConfig.Channel is EthernetListener)
                {
                    channel = PLCFactory.GetChannel((PlcComConfig.Channel as EthernetListener).LocalPort);
                }
                else
                {
                    System.Diagnostics.Debug.Assert(false);
                }

                bool isConnected = false;
                if (channel != null)
                {
                    isConnected = channel.Connected;
                }

                channel = PlcComConfig.Channel;

                PLC plc;
                if (!PlcComConfig.RequirePlcName)
                {
                     plc = PLCFactory.GetPLC(channel, PlcComConfig.UnitID);
                }
                else
                {
                    plc = PLCFactory.GetPLC(channel, PlcComConfig.UnitID, PlcComConfig.PlcName);
                }

                string plcName = "";

                try
                {
                    plcName = plc.PlcName;
                }
                catch 
                {
                }

                PlcVersion plcVersion = plc.Version;

                if (!isConnected)
                    plc.Disconnect();

                if (OnGetPlcInformation != null)
                    OnGetPlcInformation(new object[] { plcVersion, plcName }, null);

                if (OnGetPlcInformationFinished != null)
                    OnGetPlcInformationFinished(true);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
                if (OnGetPlcInformationFinished != null)
                    OnGetPlcInformationFinished(false);
            }
            finally
            {
                m_GuiState.BusyCursorVisible = false;
                m_GuiState.GetPlcInformationButtonEnabled = true;

                if (OnGuiStateChanged != null)
                    OnGuiStateChanged(m_GuiState, null);
            }
        }

        internal class GetPlcInformationEventArgs : EventArgs
        {
            public PlcVersion PlcVersion;
        }
    }
}
