﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Unitronics.ComDriver;
using FileTypeAndIcon;
using Infragistics.Win.UltraWinToolbars;

namespace Unitronics.SdCardExplorer
{
    [ToolboxBitmap(typeof(resfinder), "Unitronics.SdCardExplorer.Resources.ToolboxBitmap.png")]
    public partial class SdCardExplorer : UserControl
    {
        public enum enWorkingMode
        {
            FileSelection,
            SdBrowsing,
        }

        private enWorkingMode m_WorkingMode;
        private ExplorerPresenter m_BrowserPresenter;
        private string m_SelectedDir = "";
        private string folderDialogSelectedPath = "";
        ExplorerPresenter.GuiEnableState m_guiEnableState;

        #region constructor
        public SdCardExplorer()
        {
            InitializeComponent();
            m_BrowserPresenter = new ExplorerPresenter(this);
            m_BrowserPresenter.OnNotify += new EventHandler(m_BrowserPresenter_OnNotify);
            m_BrowserPresenter.OnGuiEnableStateChanged += new EventHandler(m_BrowserPresenter_OnGuiEnableStateChanged);
            m_BrowserPresenter.OnDirRelist += new EventHandler(m_BrowserPresenter_OnDirRelist);
            m_BrowserPresenter.OnBusyCursorVisibleChange += new EventHandler(m_BrowserPresenter_OnBusyCursorVisibleChange);

            this.BusyCursor.BringToFront();
            this.BusyCursor.Visible = false;
            m_guiEnableState = new ExplorerPresenter.GuiEnableState();
            ultraToolbarsManager1.Tools["btnAbort"].SharedProps.Enabled = false;
            ultraToolbarsManager1.Tools["btnCopy"].SharedProps.Enabled = false;
            ultraToolbarsManager1.Tools["btnDeleteFile"].SharedProps.Enabled = false;
            ultraToolbarsManager1.Tools["btnDownloadFile"].SharedProps.Enabled = false;
            ultraToolbarsManager1.Tools["btnUploadFile"].SharedProps.Enabled = false;
            ultraToolbarsManager1.Tools["btnPaste"].SharedProps.Enabled = false;
            ultraToolbarsManager1.Tools["btnRefresh"].SharedProps.Enabled = false;     
            this.WorkingMode = enWorkingMode.SdBrowsing;

            ListTool viewsList = this.ultraToolbarsManager1.Tools["ViewsList"] as ListTool;
            if (viewsList != null)
                viewsList.SelectedItemIndex = viewsList.ListToolItems["Details"].Index;
        }

        #endregion

        #region properties

        [Browsable(false)]
        public PLC Plc
        {
            set
            {
                m_BrowserPresenter.Plc = value;
            }
            get
            {
                return m_BrowserPresenter.Plc;
            }

        }

        public string SelectedFile()
        {
            if ((treeFolders.SelectedNode != null) && (listFiles.SelectedItems.Count > 0))
            {
                string pathAsString = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);
                pathAsString += "\\" + listFiles.SelectedItems[0].Text;
                return pathAsString;
            }
            return "";
        }

        public enWorkingMode WorkingMode
        {
            set
            {
                m_WorkingMode = value;
                if (m_WorkingMode == enWorkingMode.FileSelection)
                {
                    this.ultraToolbarsManager1.Toolbars["ultraToolbar2"].Visible = false;
                    listFiles.MultiSelect = false;
                    listFiles.AllowDrop = false;
                }
                else
                {
                    this.ultraToolbarsManager1.Toolbars["ultraToolbar2"].Visible = true;
                    listFiles.MultiSelect = true;
                    listFiles.AllowDrop = true;
                }
            }
            get
            {
                return m_WorkingMode;
            }
        }

        public SD.SdFolder HomeDirectory
        {
            set
            {
                m_BrowserPresenter.HomeDirectory = value;
            }
            get
            {
                return m_BrowserPresenter.HomeDirectory;
            }
        }

        #endregion

        #region public methods
        protected void UpdateUI(MethodInvoker uiDelegate)
        {
            if (InvokeRequired)
                this.Invoke(uiDelegate);
            else
                uiDelegate();
        }

        #endregion

        #region events
        private void Browser_Resize(object sender, EventArgs e)
        {
            this.BusyCursor.Left = (this.Width - this.BusyCursor.Width) / 2;
            this.BusyCursor.Top = (this.Height - this.BusyCursor.Height) / 2;
        }

        private void listFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            ultraToolbarsManager1.Tools["btnDeleteFile"].SharedProps.Enabled = m_guiEnableState.Delete && checkDeleteEnableState();
            ultraToolbarsManager1.Tools["btnDownloadFile"].SharedProps.Enabled = m_guiEnableState.Download && checkDownloadEnableState();
        }

        private void timerCheckPaste_Tick(object sender, EventArgs e)
        {
            ultraToolbarsManager1.Tools["btnPaste"].SharedProps.Enabled = m_guiEnableState.Paste && checkPasteEnableState();
        }

        private void ViewsClicked(object sender, ToolClickEventArgs e)
        {
            // Change the view of the listview based on the user's selection.
            if (e.ListToolItem != null)
            {
                this.listFiles.BeginUpdate();

                switch (e.ListToolItem.Key)
                {
                    case "Icons":
                        this.listFiles.View = View.LargeIcon;
                        break;

                    case "List":
                        this.listFiles.View = View.List;
                        break;

                    case "Details":
                        this.listFiles.View = View.Details;
                        break;
                }

                this.listFiles.EndUpdate();
            }
        }

        private void treeFolders_AfterSelect(object sender, TreeViewEventArgs e)
        {
            string dir = e.Node.FullPath;
            m_SelectedDir = dir; // Will be used later after relisting the dirs to go back to that directory/node
            SD.Folder folder = m_BrowserPresenter.GetSdFolderFromPath(dir);

            listFiles.BeginUpdate();
            listFiles.Items.Clear();

            foreach (SD.Folder subFolder in folder.Folders)
            {
                ListViewItem folderDetails = new ListViewItem(new string[] { subFolder.Name, "", "File Folder", subFolder.DateModified.ToString() });
                folderDetails.Tag = subFolder;

                string pathAsString = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);

                pathAsString += "\\" + subFolder.Name;
                bool isUnitronicsDir = m_BrowserPresenter.isUnitronicsDirectory(pathAsString);

                if (isUnitronicsDir)
                {
                    folderDetails.ImageKey = "unlockedFolder.png";
                }
                else
                {
                    folderDetails.ImageKey = "lockedFolder.png";
                }
                
                listFiles.Items.Add(folderDetails);
            }

            foreach (SD.File file in folder.Files)
            {
                string extension = System.IO.Path.GetExtension(file.Name);
                if (extension != "")
                    extension = System.IO.Path.GetExtension(file.Name).Substring(1).ToUpper();

                ListViewItem fileDetails = new ListViewItem(new string[] { file.Name, ExplorerPresenter.FileSizeAsString(file.Size), 
                    extension + " File", file.DateModified.ToString() });

                fileDetails.Tag = file;
                if (smallImageList.Images.ContainsKey(extension))
                {
                    fileDetails.ImageKey = extension;
                }
                else
                {
                    Icon[] icons = RegisteredFileType.GetIconFromExtension(System.IO.Path.GetExtension(file.Name));

                    if ((icons == null) || (icons[0] == null) || (icons[1] == null))
                        icons = RegisteredFileType.GetShellIconsFromExtension(System.IO.Path.GetExtension(file.Name));

                    if (icons != null)
                    {
                        if (icons[0] != null || icons[1] != null)
                        {
                            if ((icons[0] != null) && (icons[0].Width != 0) && (icons[0].Height != 0))
                            {
                                largeImageList.Images.Add(extension, icons[0]);
                            }
                            else
                            {
                                largeImageList.Images.Add(extension, icons[1]);
                            }

                            if ((icons[1] != null) && (icons[1].Width != 0) && (icons[1].Height != 0))
                            {
                                smallImageList.Images.Add(extension, icons[1]);
                            }
                            else
                            {
                                smallImageList.Images.Add(extension, icons[0]);
                            }
                            fileDetails.ImageKey = extension;
                        }
                        else
                        {
                            fileDetails.ImageKey = "File.png";
                        }   
                    }
                    else
                    {
                        fileDetails.ImageKey = "File.png";
                    }
                }
                
                listFiles.Items.Add(fileDetails);
            }
            listFiles.EndUpdate();

            ultraToolbarsManager1.Tools["btnDeleteFile"].SharedProps.Enabled = m_guiEnableState.Delete && checkDeleteEnableState();
            ultraToolbarsManager1.Tools["btnDownloadFile"].SharedProps.Enabled = m_guiEnableState.Download && checkDownloadEnableState();

        }

        private void updateFoldersAndFilesLists(SD.Folder folder, TreeNode parentNode, bool addToTree)
        {
            foreach (SD.Folder subFolder in folder.Folders)
            {
                TreeNode subNode = new TreeNode(subFolder.Name);
                subNode.Text = subFolder.Name;
                subNode.Name = subFolder.Name;
                parentNode.Nodes.Add(subNode);

                if (addToTree)
                    treeFolders.Nodes.Add(subNode);

                string pathAsString = subNode.FullPath;
                bool isUnitronicsDir = m_BrowserPresenter.isUnitronicsDirectory(pathAsString);

                if (isUnitronicsDir)
                {
                    subNode.ImageKey = "unlockedFolder.png";
                    subNode.SelectedImageKey = "unlockedFolder.png";
                }
                else
                {
                    subNode.ImageKey = "lockedFolder.png";
                    subNode.SelectedImageKey = "lockedFolder.png";
                }

                updateFoldersAndFilesLists(subFolder, subNode, false);
            }
        }

        void m_BrowserPresenter_OnDirRelist(object sender, EventArgs e)
        {
            SD.Folder folder = sender as SD.Folder;
            if (folder != null)
            {
                MethodInvoker uiDelegate = delegate
                {
                    treeFolders.BeginUpdate();
                    treeFolders.Nodes.Clear();
                    TreeNode parentNode = new TreeNode();
                    updateFoldersAndFilesLists(folder, parentNode, true);

                    treeFolders.EndUpdate();
                    treeFolders.Refresh();
                    listFiles.Items.Clear();

                    if (treeFolders.Nodes.Count > 0)
                    {
                        if (m_SelectedDir == "")
                        {
                            treeFolders.SelectedNode = treeFolders.Nodes[0];
                        }
                        else
                        {
                            string[] subFolders = m_SelectedDir.Split('\\');

                            try
                            {
                                parentNode = treeFolders.Nodes[subFolders[0]];
                            }
                            catch { }

                            for (int i = 1; i < subFolders.Length; i++)
                            {
                                try
                                {
                                    parentNode = parentNode.Nodes[subFolders[i]];
                                }
                                catch
                                {
                                }
                            }

                            if (parentNode != null)
                            {
                                treeFolders.SelectedNode = parentNode;
                            }
                            else
                            {
                                treeFolders.SelectedNode = treeFolders.Nodes[0];
                            }
                        }
                    }
                };
                UpdateUI(uiDelegate);
            }
        }

        void m_BrowserPresenter_OnGuiEnableStateChanged(object sender, EventArgs e)
        {
            ExplorerPresenter.GuiEnableState guiEnableState = sender as ExplorerPresenter.GuiEnableState;
            m_guiEnableState = guiEnableState;
            if (guiEnableState != null)
            {
                MethodInvoker uiDelegate = delegate
                {
                    ultraToolbarsManager1.Tools["btnAbort"].SharedProps.Enabled = guiEnableState.Abort;
                    ultraToolbarsManager1.Tools["btnCopy"].SharedProps.Enabled = guiEnableState.Copy;
                    ultraToolbarsManager1.Tools["btnDeleteFile"].SharedProps.Enabled = guiEnableState.Delete && checkDeleteEnableState();
                    ultraToolbarsManager1.Tools["btnDownloadFile"].SharedProps.Enabled = guiEnableState.Download && checkDownloadEnableState(); 
                    ultraToolbarsManager1.Tools["btnUploadFile"].SharedProps.Enabled = guiEnableState.Upload;
                    ultraToolbarsManager1.Tools["btnPaste"].SharedProps.Enabled = guiEnableState.Paste && checkPasteEnableState();
                    ultraToolbarsManager1.Tools["btnRefresh"].SharedProps.Enabled = guiEnableState.Refresh;
                    treeFolders.Enabled = guiEnableState.Folders;
                };
                UpdateUI(uiDelegate);
            }
        }

        void m_BrowserPresenter_OnNotify(object sender, EventArgs e)
        {
            ExplorerPresenter.Notifiy notify = sender as ExplorerPresenter.Notifiy;
            if (notify != null)
            {
                MethodInvoker uiDelegate = delegate
                {
                    ultraStatusBar1.Panels["ActionText"].Text = notify.StatusText;
                    ultraStatusBar1.Panels["ActionText"].Appearance.ForeColor = notify.StatusTextColor;
                    ultraStatusBar1.Panels["ProgressBar"].ProgressBarInfo.Minimum = notify.ProgressMin;
                    ultraStatusBar1.Panels["ProgressBar"].ProgressBarInfo.Maximum = notify.ProgressMax;
                    ultraStatusBar1.Panels["ProgressBar"].ProgressBarInfo.Value = notify.ProgressValue;
                    ultraStatusBar1.Panels["ProgressBar"].Visible = notify.ShowProgress;
                };
                UpdateUI(uiDelegate);
            }
        }

        void m_BrowserPresenter_OnBusyCursorVisibleChange(object sender, EventArgs e)
        {
            bool busyCursorVisibility = (bool)sender;
            MethodInvoker uiDelegate = delegate
            {
                BusyCursor.Visible = busyCursorVisibility;
            };
            UpdateUI(uiDelegate);
        }

        private void ultraToolbarsManager1_ToolClick(object sender, Infragistics.Win.UltraWinToolbars.ToolClickEventArgs e)
        {
            switch (e.Tool.Key)
            {
                case "btnRefresh":
                    reloadFilesList();
                    break;
                case "btnAbort":
                    abort();
                    break;
                case "btnDownloadFile":
                    downloadFilesAndFolders();
                    break;
                case "btnUploadFile":
                    uploadFiles();
                    break;
                case "btnDeleteFile":
                    deleteFiles();
                    break;
                case "btnCopy":
                    copyFiles();
                    break;
                case "btnPaste":
                    pasteFiles();
                    break;
                case "ViewsList":
                    ViewsClicked(sender, e);
                    break;

            }
        }

        private void listFiles_DoubleClick(object sender, EventArgs e)
        {
            if (!m_guiEnableState.Files)
                return;

            try
            {
                if ((listFiles.SelectedItems.Count > 0) && (listFiles.SelectedItems[0].Tag is SD.Folder))
                {
                    string folder = listFiles.SelectedItems[0].Text;
                    treeFolders.SelectedNode = treeFolders.SelectedNode.Nodes[folder];
                }
                else if (listFiles.SelectedItems.Count > 0)
                {
                    // double clicked on a file

                    if (m_WorkingMode == enWorkingMode.FileSelection)
                        return;

                    List<SD.File> fileNames = new List<SD.File>();
                    List<SD.Folder> folderNames = new List<SD.Folder>();

                    string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);

                    fileNames.Add((SD.File)listFiles.SelectedItems[0].Tag);
                    string destPath = System.IO.Path.GetTempPath();

                    if (destPath[destPath.Length - 1] == '\\')
                    {
                        destPath = destPath.Substring(0, destPath.Length - 1);
                    }

                    ExplorerPresenter.downloadCompleteDelagate downloadCompletedDel = new ExplorerPresenter.downloadCompleteDelagate(openFileAfterDownload);
                    m_BrowserPresenter.DownloadFilesAndFolders(dir, fileNames, folderNames, destPath, false, downloadCompletedDel);
                }
            }
            catch
            {
            }
        }

        private void filesContextMenu_Opening(object sender, CancelEventArgs e)
        {
            if ((listFiles.SelectedItems.Count == 0) || (!m_guiEnableState.Files))
            {
                e.Cancel = true;
                return;
            }

            filesContextMenu.Items["readSelectedFilesAndFoldersToolStripMenuItem"].Enabled = checkDownloadEnableState();
            filesContextMenu.Items["deleteSelectedFilesToolStripMenuItem"].Enabled = checkDeleteEnableState();
        }

        private void fileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem item = sender as ToolStripMenuItem;

            switch (item.Name)
            {
                case "readSelectedFilesAndFoldersToolStripMenuItem":
                    downloadFilesAndFolders();
                    break;
                case "deleteSelectedFilesToolStripMenuItem":
                    deleteFiles();
                    break;
                case "propertiesToolStripMenuItem":
                    int numOfFiles;
                    int numOfFolders;
                    long totalSize;
                    getSelectedFilesAndFoldersParams(out numOfFiles, out numOfFolders, out totalSize);
                    frmFilesProperties filesProperties = new frmFilesProperties();
                    filesProperties.NumOfFiles = numOfFiles;
                    filesProperties.NumOfFoldres = numOfFolders;
                    filesProperties.Size = totalSize;
                    filesProperties.FilesLocation = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);
                    filesProperties.ShowDialog();
                    break;
                default:
                    break;
            }
        }

        private void listFiles_KeyUp(object sender, KeyEventArgs e)
        {
            if (!m_guiEnableState.Files)
                return;

            try
            {
                if ((listFiles.SelectedItems.Count > 0) && (listFiles.SelectedItems[0].Tag is SD.Folder) && (e.KeyValue == 13))
                {
                    string folder = listFiles.SelectedItems[0].Text;
                    treeFolders.SelectedNode = treeFolders.SelectedNode.Nodes[folder];
                }
                else if ((e.KeyValue == 'V') && (e.Control) && (!e.Alt) && (!e.Shift))
                {
                    if (m_WorkingMode == enWorkingMode.FileSelection)
                        return;

                    pasteFiles();
                }
                else if ((e.KeyValue == 'A') && (e.Control) && (!e.Alt) && (!e.Shift))
                {
                    if (m_WorkingMode == enWorkingMode.FileSelection)
                        return;

                    listFiles.Focus();
                    SendKeys.Send("{HOME} ");
                    SendKeys.Send("+{END}");

                }
                else if ((e.KeyValue == 46) && (!e.Control) && (!e.Alt) && (!e.Shift))
                {
                    if (m_WorkingMode == enWorkingMode.FileSelection)
                        return;

                    deleteFiles();
                }
            }
            catch
            {
            }
        }

        private void treeFolders_KeyUp(object sender, KeyEventArgs e)
        {
            if (!m_guiEnableState.Files)
                return;

            try
            {
                if ((listFiles.SelectedItems.Count > 0) && (listFiles.SelectedItems[0].Tag is SD.Folder) && (e.KeyValue == 13))
                {
                    string folder = listFiles.SelectedItems[0].Text;
                    treeFolders.SelectedNode = treeFolders.SelectedNode.Nodes[folder];
                }
                else if ((e.KeyValue == 'V') && (e.Control) && (!e.Alt) && (!e.Shift))
                {
                    if (m_WorkingMode == enWorkingMode.FileSelection)
                        return;

                    pasteFiles();
                }
            }
            catch
            {
            }
        }

        private void listFiles_DragDrop(object sender, DragEventArgs e)
        {
            if (m_WorkingMode == enWorkingMode.FileSelection)
                return;

            if (!m_guiEnableState.Files)
                return;

            // transfer the filenames to a string array
            // (yes, everything to the left of the "=" can be put in the 
            // foreach loop in place of "files", but this is easier to understand.)
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);

            if (treeFolders.SelectedNode == null)
                return;

            if (files == null)
                return;

            if (files.Count() == 0)
                return;

            string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);

            List<string> fileNames = new List<string>();

            foreach (string file in files)
            {
                if (System.IO.Path.GetExtension(file) != "")
                {
                    fileNames.Add(file);
                }
            }


            List<string> randomFiles = new List<string>();

            Random random = new Random();

            while (fileNames.Count > 0)
            {
                int randomNumber = random.Next(0, fileNames.Count);
                string file = fileNames[randomNumber];
                fileNames.Remove(file);
                randomFiles.Add(file);
            }

            fileNames = randomFiles;

            if (fileNames.Count() > 0)
                m_BrowserPresenter.UploadFiles(fileNames, dir);
        }

        private void listFiles_DragEnter(object sender, DragEventArgs e)
        {
            // make sure they're actually dropping files (not text or anything else)
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
                // allow them to continue
                // (without this, the cursor stays a "NO" symbol
                e.Effect = DragDropEffects.All;
        }

        #endregion
            
        private void reloadFilesList()
        {
            m_BrowserPresenter.ReloadFilesList();
        }

        private void abort()
        {
            m_BrowserPresenter.Abort();
        }

        private void downloadFilesAndFolders()
        {
            if (treeFolders.SelectedNode == null)
                return;

            if (!m_guiEnableState.Files)
                return;

            var files = listFiles.SelectedItems;  
            if (files.Count == 0)
                return;

            FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
            folderBrowserDialog.SelectedPath = folderDialogSelectedPath;

            if (folderBrowserDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            folderDialogSelectedPath = folderBrowserDialog.SelectedPath;

            List<SD.File> fileNames = new List<SD.File>();
            List<SD.Folder> folderNames = new List<SD.Folder>();

            string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);

            foreach (ListViewItem item in files)
            {
                if (item.Tag is SD.Folder)
                {
                    folderNames.Add((SD.Folder)item.Tag);
                }
                else
                {
                    fileNames.Add((SD.File)item.Tag);
                }
            }

            m_BrowserPresenter.DownloadFilesAndFolders(dir, fileNames, folderNames, folderBrowserDialog.SelectedPath, true, null);

            frmConfirmFileReplace confirmReplace = new frmConfirmFileReplace();
        }

        private void getSelectedFilesAndFoldersParams(out int numOfFiles, out int numOfFolders, out long totalSize)
        {
            numOfFiles = 0;
            numOfFolders = 0;
            totalSize = 0;

            List<SD.File> fileNames = new List<SD.File>();
            List<SD.Folder> folderNames = new List<SD.Folder>();

            string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);
            var files = listFiles.SelectedItems;

            foreach (ListViewItem item in files)
            {
                if (item.Tag is SD.Folder)
                {
                    folderNames.Add((SD.Folder)item.Tag);
                }
                else
                {
                    fileNames.Add((SD.File)item.Tag);
                }
            }

            getFilesParams(fileNames, ref numOfFiles, ref numOfFolders, ref totalSize);
            getFoldersParams(folderNames, ref numOfFiles, ref numOfFolders, ref totalSize);
        }

        private void getFilesParams(List<SD.File> files, ref int numOfFiles, ref int numOfFolders, ref long totalSize)
        {
            foreach (SD.File file in files)
            {
                numOfFiles++;
                totalSize += file.Size;
            }
        }

        private void getFoldersParams(List<SD.Folder> folders, ref int numOfFiles, ref int numOfFolders, ref long totalSize)
        {
            foreach (SD.Folder folder in folders)
            {
                numOfFolders++;
                getFilesParams(folder.Files, ref numOfFiles, ref numOfFolders, ref totalSize);
                getFoldersParams(folder.Folders, ref numOfFiles, ref numOfFolders, ref totalSize);
            }
        }

        private void uploadFiles()
        {
            if (treeFolders.SelectedNode == null)
                return;

            OpenFileDialog openFileDialog = new OpenFileDialog();
            openFileDialog.Multiselect = true;

            if (openFileDialog.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            if (openFileDialog.FileNames.Count() == 0)
            {
                return;
            }

            string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);

            List<string> fileNames = openFileDialog.FileNames.ToList();

            List<string> randomFiles = new List<string>();

            Random random = new Random();

            while (fileNames.Count > 0)
            {
                int randomNumber = random.Next(0, fileNames.Count);
                string file = fileNames[randomNumber];
                fileNames.Remove(file);
                randomFiles.Add(file);
            }

            fileNames = randomFiles;

            m_BrowserPresenter.UploadFiles(fileNames, dir);
        }

        private void deleteFiles()
        {
            bool isReadonly = false;
            if (treeFolders.SelectedNode == null)
                return;

            if (!m_guiEnableState.Files)
                return;

            var files = listFiles.SelectedItems;
            if (files.Count == 0)
                return;

            DialogResult dialogResult = MessageBox.Show("Are you sure you want to delete the selected file(s)?", "Confirm File(s) delete",MessageBoxButtons.YesNo);

            if (dialogResult == DialogResult.No)
                return;

            List<string> fileNames = new List<string>();

            string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);
            foreach (ListViewItem item in files)
            {
                // Only deleting files
                // SD.Folder derives from SD.File, so I cannot ask is SD.File cause it always gives True
                if (!(item.Tag is SD.Folder))
                {
                    string file = dir + @"\" + item.Text;
                    fileNames.Add(file);
                }
            }

            m_BrowserPresenter.DeleteFiles(fileNames);
            
        }

        private void copyFiles()
        {
            if (treeFolders.SelectedNode == null)
                return;

            if (!m_guiEnableState.Files)
                return;

            var files = listFiles.SelectedItems;  
            if (files.Count == 0)
                return;

            List<string> fileNames = new List<string>();

            string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);
            foreach (ListViewItem item in files)
            {
                // Only downloading files
                // SD.Folder derives from SD.File, so I cannot ask is SD.File cause it always gives True
                if (!(item.Tag is SD.Folder))
                {
                    string file = dir + @"\" + item.Text;
                    fileNames.Add(file);
                }
            }

            Clipboard.SetData(DataFormats.FileDrop, fileNames.ToArray());
        }

        private void pasteFiles()
        {
            if (treeFolders.SelectedNode == null)
                return;

            if (!m_guiEnableState.Files)
                return;

            string[] files = (string[])Clipboard.GetData(DataFormats.FileDrop);

            if (files == null)
                return;

            if (files.Count() == 0)
                return;

            string dir = ExplorerPresenter.GetFullPathFromHomeDirAndPath(HomeDirectory, treeFolders.SelectedNode.FullPath);

            List<string> fileNames = files.ToList();

            List<string> randomFiles = new List<string>();

            Random random = new Random();

            while (fileNames.Count > 0)
            {
                int randomNumber = random.Next(0, fileNames.Count);
                string file = fileNames[randomNumber];
                fileNames.Remove(file);
                randomFiles.Add(file);
            }

            fileNames = randomFiles;

            m_BrowserPresenter.UploadFiles(fileNames, dir);
        }

        private void openFileAfterDownload(string currentDir, List<SD.File> files, List<SD.Folder> folders, string destPath)
        {
            if (files.Count() == 0)
                return;

            System.Diagnostics.Process proc = new System.Diagnostics.Process();
            proc.EnableRaisingEvents = false;
            proc.StartInfo.FileName = destPath + "\\" + files[0].Name;
            proc.Start();
        }

        private bool checkDownloadEnableState()
        {
            if (listFiles.SelectedItems.Count > 0)
            {
                return true;
            }
            return false;
        }

        private bool checkDeleteEnableState()
        {
            
            if (listFiles.SelectedItems.Count == 0)
            {
                return false;
            }
            foreach (ListViewItem item in listFiles.SelectedItems)
            {
                // SD.Folder derives from SD.File, so I cannot ask is SD.File cause it always gives True
                if (!(item.Tag is SD.Folder))
                {
                    return true;
                }
            } return false;
        }

        private bool checkPasteEnableState()
        {
            if (treeFolders.SelectedNode == null)
                return false;

            string[] files = (string[])Clipboard.GetData(DataFormats.FileDrop);

            if (files == null)
                return false;

            if (files.Count() == 0)
                return false;

            foreach (string file in files)
            {
                if (System.IO.Path.GetExtension(file) != "")
                {
                    return true;
                }
            }

            return false;
        }

    }
}

internal class resfinder { }
