﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;


namespace Unitronics.SdCardExplorer.Lite
{
    internal partial class frmConfirmFileReplace : Form
    {
        private System.Windows.Forms.ToolTip m_tooltip;

        public enum ConfirmReplaceButtons
        {
            None = 0,
            Overwrite = 1,
            OverwriteAll = 2,
            Resume = 4,
            ResumeAll = 8,
            Skip = 16,
            SkipAll = 32,
            Cancel = 64,
        }

        private ConfirmReplaceButtons m_Result;

        public string FileName
        {
            set
            {
                lblPrefix.Text = "This folder already contains a file named \"" + value + "\"";
            }
        }

        public long SizeExisting
        {
            set
            {
                lblSizeExisting.Text = string.Format("{0:0,0}",value) + " Bytes";
            }
        }

        public string DateExisting
        {
            set
            {
                lblDateExisting.Text = value;
            }
        }

        public long SizeNew
        {
            set
            {
                lblSizeNew.Text = string.Format("{0:0,0}", value) + " Bytes";
            }
        }

        public string DateNew
        {
            set
            {
                lblDateNew.Text = value;
            }
        }

        public ConfirmReplaceButtons Result
        {
            get
            {
                return m_Result;
            }
            internal set
            {
                m_Result = value;
            }
        }

        public int enabledButtons
        {
            set
            {
                btnOverwrite.Enabled = ((int)ConfirmReplaceButtons.Overwrite & value) > 0;
                btnOverwriteAll.Enabled = ((int)ConfirmReplaceButtons.OverwriteAll & value) > 0;
                btnResume.Enabled = ((int)ConfirmReplaceButtons.Resume & value) > 0;
                btnResumeAll.Enabled = ((int)ConfirmReplaceButtons.ResumeAll & value) > 0;
                btnSkip.Enabled = ((int)ConfirmReplaceButtons.Skip & value) > 0;
                btnSkipAll.Enabled = ((int)ConfirmReplaceButtons.SkipAll & value) > 0;
                btnCancel.Enabled = ((int)ConfirmReplaceButtons.Cancel & value) > 0;
            }
        }

        public Image FileIcon
        {
            set
            {
                if (value != null)
                {
                    picIcon1.Image = value;
                    picIcon2.Image = picIcon1.Image;
                }
                else
                {
                    picIcon1.Image = largeImageList.Images["File.png"];
                }
            }
        }

        public frmConfirmFileReplace()
        {
            InitializeComponent();
            m_tooltip = new ToolTip();
            Result = ConfirmReplaceButtons.None;
            btnOverwrite.Tag = ConfirmReplaceButtons.Overwrite;
            btnOverwriteAll.Tag = ConfirmReplaceButtons.OverwriteAll;
            btnResume.Tag = ConfirmReplaceButtons.Resume;
            btnResumeAll.Tag = ConfirmReplaceButtons.ResumeAll;
            btnSkip.Tag = ConfirmReplaceButtons.Skip;
            btnSkipAll.Tag = ConfirmReplaceButtons.SkipAll;
            btnCancel.Tag = ConfirmReplaceButtons.Cancel;

            m_tooltip.SetToolTip(this.btnOverwrite, "Overwrite current file");
            m_tooltip.SetToolTip(this.btnOverwriteAll, "Overwrite all files");

            m_tooltip.SetToolTip(this.btnResume, "Resume current file");
            m_tooltip.SetToolTip(this.btnResumeAll, "Resume all files in which the target file is smaller than the source file");

            m_tooltip.SetToolTip(this.btnSkip, "Skip current file");
            m_tooltip.SetToolTip(this.btnSkipAll, "Skip all files if target file already exists");

            m_tooltip.SetToolTip(this.btnCancel, "Cancel all files");
        }

        private void button_Click(object sender, EventArgs e)
        {
            Result = (ConfirmReplaceButtons)((sender as Button).Tag);
            this.Close();
        }

        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams cp = base.CreateParams;
                const int CS_NOCLOSE = 0x200;
                cp.ClassStyle |= CS_NOCLOSE;
                return cp;
            }
        }
    }
}
